diff --git a/MySqlPlus.Example/Program.cs b/MySqlPlus.Example/Program.cs
index cd9bfcd..75dd511 100644
--- a/MySqlPlus.Example/Program.cs
+++ b/MySqlPlus.Example/Program.cs
@@ -10,7 +10,7 @@ namespace MontoyaTech.MySqlPlus.Example
[MySqlRowIndex("year", "year")]
public class Car
{
- [MySqlColumn(Id = true, Name = "id", PrimaryKey = true, AutoIncrement = true)]
+ [MySqlColumn(Name = "id", PrimaryKey = true, AutoIncrement = true)]
public ulong Id = 0;
[MySqlColumn("make")]
diff --git a/MySqlPlus/MySqlColumnExtensions.cs b/MySqlPlus/MySqlColumnExtensions.cs
index de68981..2a58711 100644
--- a/MySqlPlus/MySqlColumnExtensions.cs
+++ b/MySqlPlus/MySqlColumnExtensions.cs
@@ -22,11 +22,11 @@ namespace MontoyaTech.MySqlPlus
///
///
///
- public static bool GetMySqlId(this T row, out FieldInfo field, out MySqlColumn idColumn)
+ public static bool GetMySqlPrimaryKey(this T row, out FieldInfo field, out MySqlColumn idColumn)
{
var type = typeof(T);
- return GetMySqlId(type, out field, out idColumn);
+ return GetMySqlPrimaryKey(type, out field, out idColumn);
}
///
@@ -37,7 +37,7 @@ namespace MontoyaTech.MySqlPlus
///
///
///
- public static bool GetMySqlId(this Type type, out FieldInfo field, out MySqlColumn idColumn)
+ public static bool GetMySqlPrimaryKey(this Type type, out FieldInfo field, out MySqlColumn idColumn)
{
field = null;
@@ -52,7 +52,7 @@ namespace MontoyaTech.MySqlPlus
{
var column = fields[i].GetCustomAttribute();
- if (column != null && column.Id)
+ if (column != null && column.PrimaryKey)
{
field = fields[i];
idColumn = column;
@@ -246,6 +246,10 @@ namespace MontoyaTech.MySqlPlus
///
public static bool IsNullable(this MySqlColumn column, FieldInfo field, T row)
{
+ //If the column is the primary key, it cannot be null.
+ if (column.PrimaryKey)
+ return false;
+
var fieldType = field.FieldType;
var defaultValue = field.GetValue(row);
diff --git a/MySqlPlus/MySqlCommandExtensions.cs b/MySqlPlus/MySqlCommandExtensions.cs
index 4a85309..e8d10e6 100644
--- a/MySqlPlus/MySqlCommandExtensions.cs
+++ b/MySqlPlus/MySqlCommandExtensions.cs
@@ -51,8 +51,8 @@ namespace MontoyaTech.MySqlPlus
if (column == null)
continue;
- //Skip id columns because they are auto incremented.
- if (column.Id)
+ //Skip any columns that are auto incremented.
+ if (column.AutoIncrement)
continue;
if (seperate)
@@ -99,8 +99,8 @@ namespace MontoyaTech.MySqlPlus
//Write the set values section
builder.Append("SET ");
- FieldInfo idField = null;
- MySqlColumn idColumn = null;
+ FieldInfo primaryField = null;
+ MySqlColumn primaryColumn = null;
bool seperate = false;
for (int i = 0; i < fields.Length; i++)
{
@@ -109,14 +109,17 @@ namespace MontoyaTech.MySqlPlus
if (column == null)
continue;
- //Skip id columns because they are auto incremented.
- if (column.Id)
+ //If this column is the primary key, grab it.
+ if (column.PrimaryKey)
{
- idField = fields[i];
- idColumn = column;
- continue;
+ primaryField = fields[i];
+ primaryColumn = column;
}
+ //Skip any columns that are auto incremented.
+ if (column.AutoIncrement)
+ continue;
+
if (seperate)
builder.Append(", ");
@@ -127,14 +130,14 @@ namespace MontoyaTech.MySqlPlus
seperate = true;
}
- //Make sure we have an id column.
- if (idField == null)
- throw new Exception("Given row does not contain an id column");
+ //Make sure we have a primary key.
+ if (primaryField == null)
+ throw new Exception("Given row does not contain primary key.");
//Write the where clause
- builder.Append($" WHERE `{(string.IsNullOrWhiteSpace(idColumn.Name) ? idField.Name : idColumn.Name)}`=@id LIMIT 1");
+ builder.Append($" WHERE `{(string.IsNullOrWhiteSpace(primaryColumn.Name) ? primaryField.Name : primaryColumn.Name)}`=@id LIMIT 1");
- command.Parameters.AddWithValue("@id", idField.GetValue(row));
+ command.Parameters.AddWithValue("@id", primaryField.GetValue(row));
//Set the command text.
command.CommandText = builder.ToString();
@@ -156,11 +159,11 @@ namespace MontoyaTech.MySqlPlus
var rowAttribute = rowType.GetCustomAttribute();
//Get the id field
- if (!rowType.GetMySqlId(out FieldInfo idField, out MySqlColumn idColumn))
- throw new Exception("Failed to find the id column on the given row.");
+ if (!rowType.GetMySqlPrimaryKey(out FieldInfo primaryField, out MySqlColumn primaryColumn))
+ throw new Exception("Failed to find the primary key for the given row.");
//Set the command text.
- command.CommandText = $"SELECT * FROM `{(rowAttribute == null || string.IsNullOrWhiteSpace(rowAttribute.Name) ? rowType.Name : rowAttribute.Name)}` WHERE `{(string.IsNullOrWhiteSpace(idColumn.Name) ? idField.Name : idColumn.Name)}`=@id LIMIT 1";
+ command.CommandText = $"SELECT * FROM `{(rowAttribute == null || string.IsNullOrWhiteSpace(rowAttribute.Name) ? rowType.Name : rowAttribute.Name)}` WHERE `{(string.IsNullOrWhiteSpace(primaryColumn.Name) ? primaryField.Name : primaryColumn.Name)}`=@id LIMIT 1";
//Add the id parameter.
command.Parameters.AddWithValue("@id", id);
@@ -204,15 +207,15 @@ namespace MontoyaTech.MySqlPlus
//Write the delete from section
builder.Append($"DELETE FROM `{(rowAttribute == null || string.IsNullOrWhiteSpace(rowAttribute.Name) ? rowType.Name : rowAttribute.Name)}` ");
- //Get the id the column and field info
- if (!row.GetMySqlId(out FieldInfo idField, out MySqlColumn idColumn))
- throw new Exception("Failed to find Id column on row.");
+ //Get the primary field and column
+ if (!row.GetMySqlPrimaryKey(out FieldInfo primaryField, out MySqlColumn primaryColumn))
+ throw new Exception("Failed to find primary key for given row.");
//Write the where clause
- builder.Append($"WHERE `{(string.IsNullOrWhiteSpace(idColumn.Name) ? idField.Name : idColumn.Name)}`=@id LIMIT 1");
+ builder.Append($"WHERE `{(string.IsNullOrWhiteSpace(primaryColumn.Name) ? primaryField.Name : primaryColumn.Name)}`=@id LIMIT 1");
//Add the id parameter.
- command.Parameters.AddWithValue("@id", idField.GetValue(row));
+ command.Parameters.AddWithValue("@id", primaryField.GetValue(row));
//Set the command text.
command.CommandText = builder.ToString();
@@ -238,12 +241,12 @@ namespace MontoyaTech.MySqlPlus
//Write the delete from section
builder.Append($"DELETE FROM `{(rowAttribute == null || string.IsNullOrWhiteSpace(rowAttribute.Name) ? rowType.Name : rowAttribute.Name)}` ");
- //Get the id the column and field info
- if (!rowType.GetMySqlId(out FieldInfo idField, out MySqlColumn idColumn))
- throw new Exception("Failed to find Id column on row.");
+ //Get the primary key field and column.
+ if (!rowType.GetMySqlPrimaryKey(out FieldInfo primaryField, out MySqlColumn primaryColumn))
+ throw new Exception("Failed to find primary key for the given row.");
//Write the where clause
- builder.Append($"WHERE `{(string.IsNullOrWhiteSpace(idColumn.Name) ? idField.Name : idColumn.Name)}`=@id LIMIT 1");
+ builder.Append($"WHERE `{(string.IsNullOrWhiteSpace(primaryColumn.Name) ? primaryField.Name : primaryColumn.Name)}`=@id LIMIT 1");
//Add the id parameter.
command.Parameters.AddWithValue("@id", id);
@@ -315,8 +318,10 @@ namespace MontoyaTech.MySqlPlus
if (separate)
builder.Append(", ");
+ var columnName = string.IsNullOrWhiteSpace(column.Name) ? fields[i].Name : column.Name;
+
//Write the column name
- builder.Append($"{(string.IsNullOrWhiteSpace(column.Name) ? fields[i].Name : column.Name)} ");
+ builder.Append($"{columnName} ");
//Write the column data type
builder.Append($"{column.GetMySqlColumnType(fields[i])} ");
@@ -352,8 +357,12 @@ namespace MontoyaTech.MySqlPlus
//Write the column auto increment information.
builder.Append($"{(column.AutoIncrement ? "AUTO_INCREMENT" : "")}");
+ //If the column is the primary key, output that constraint.
if (column.PrimaryKey)
- builder.Append($", PRIMARY KEY ({(string.IsNullOrWhiteSpace(column.Name) ? fields[i].Name : column.Name)})");
+ builder.Append($", PRIMARY KEY ({columnName})");
+ //If the column is unique, output that constraint, but you can't be the primary key and be unique.
+ else if (column.Unique)
+ builder.Append($", UNIQUE ({columnName})");
separate = true;
}
@@ -374,7 +383,10 @@ namespace MontoyaTech.MySqlPlus
if (separate)
builder.Append(", ");
- builder.Append($"INDEX {index.Name} ({index.Columns.Aggregate((curr, next) => curr == null ? next : curr + ", " + next)})");
+ if (string.IsNullOrWhiteSpace(index.Name))
+ builder.Append($"INDEX IDX_{index.Columns.Aggregate((curr, next) => curr == null ? next : curr + "_" + next)} ({index.Columns.Aggregate((curr, next) => curr == null ? next : curr + ", " + next)})");
+ else
+ builder.Append($"INDEX {index.Name} ({index.Columns.Aggregate((curr, next) => curr == null ? next : curr + ", " + next)})");
separate = true;
}
diff --git a/MySqlPlus/MySqlPlus.csproj b/MySqlPlus/MySqlPlus.csproj
index 4681963..daebfe3 100644
--- a/MySqlPlus/MySqlPlus.csproj
+++ b/MySqlPlus/MySqlPlus.csproj
@@ -7,7 +7,7 @@
MontoyaTech.MySqlPlus
MontoyaTech.MySqlPlus
MontoyaTech.MySqlPlus
- 1.1.1
+ 1.1.2
MontoyaTech
A simple C# library to help work with MySql.
MontoyaTech 2023
diff --git a/MySqlPlus/MySqlRowIndex.cs b/MySqlPlus/MySqlRowIndex.cs
index 7e2c929..76c2197 100644
--- a/MySqlPlus/MySqlRowIndex.cs
+++ b/MySqlPlus/MySqlRowIndex.cs
@@ -14,14 +14,14 @@ namespace MontoyaTech.MySqlPlus
public class MySqlRowIndex : Attribute
{
///
- /// The name of this index.
+ /// The name of this index. Default is null. If null, one will be created automatically.
///
- public string Name;
+ public string Name = null;
///
/// The name of the columns that belong to this index.
///
- public string[] Columns;
+ public string[] Columns = null;
///
/// Creates a new default MySqlRowIndex.
@@ -29,18 +29,15 @@ namespace MontoyaTech.MySqlPlus
public MySqlRowIndex() { }
///
- /// Creates a new default MySqlRowIndex with a name and column names.
+ /// Creates a new default MySqlRowIndex with column names apart of the index.
///
- ///
///
///
- public MySqlRowIndex(string name, params string[] columns)
+ public MySqlRowIndex(params string[] columns)
{
if (columns == null || columns.Length == 0)
throw new ArgumentException("Columns must not be null or 0.");
- this.Name = name;
-
this.Columns = columns;
}
}
diff --git a/MySqlPlus/MySqlSession.cs b/MySqlPlus/MySqlSession.cs
index 1022d5f..8df9e57 100644
--- a/MySqlPlus/MySqlSession.cs
+++ b/MySqlPlus/MySqlSession.cs
@@ -69,12 +69,13 @@ namespace MontoyaTech.MySqlPlus
this.Connection.ExecuteNonQuery(command);
- if (row.GetMySqlId(out FieldInfo idField, out MySqlColumn idColumn))
+ //Update the primary key value on the row if we can find it.
+ if (row.GetMySqlPrimaryKey(out FieldInfo primaryField, out MySqlColumn primaryColumn))
{
- if (Type.GetTypeCode(idField.FieldType) == TypeCode.UInt64)
- idField.SetValue(row, (ulong)command.LastInsertedId);
+ if (Type.GetTypeCode(primaryField.FieldType) == TypeCode.UInt64)
+ primaryField.SetValue(row, (ulong)command.LastInsertedId);
else
- idField.SetValue(row, command.LastInsertedId);
+ primaryField.SetValue(row, command.LastInsertedId);
}
return (ulong)command.LastInsertedId;
diff --git a/MySqlPlus/MySqlColumn.cs b/MySqlPlus/MySqlColumn.cs
index f274e49..935f4ae 100644
--- a/MySqlPlus/MySqlColumn.cs
+++ b/MySqlPlus/MySqlColumn.cs
@@ -17,27 +17,27 @@ namespace MontoyaTech.MySqlPlus
public class MySqlColumn : Attribute
{
///
- /// Whether or not this column is the row id.
- ///
- public bool Id = false;
-
- ///
- /// The name of the column, if null, the field name will be used instead.
+ /// The name of the column, if null, the field name will be used instead. Default is null.
///
public string Name = null;
///
- /// The MySql Data type of the column, if null, the field type will be used instead.
+ /// The MySql Data type of the column, if null, the field type will be used instead. Default is null.
///
public string Type = null;
///
- /// Whether or not this column is a primary key.
+ /// Whether or not this column is a primary key. Default is false.
///
public bool PrimaryKey = false;
///
- /// Whether or not this column auto increments.
+ /// Whether or not this column is unique. Default is false.
+ ///
+ public bool Unique = false;
+
+ ///
+ /// Whether or not this column auto increments. Default is false.
///
public bool AutoIncrement = false;